"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.reloadAuthorized = exports.getAuthMenu = exports.setAuthority = exports.getAuthority = exports.routerMatchedAuthority = exports.checkPermissions = void 0;
const base_1 = require("@teams-max/base");
const api_1 = require("../../services/api");
const { iAuthorityKey } = (0, base_1.runtimeConfig)();
// 清除权限缓存数据
localStorage.removeItem(iAuthorityKey);
/**
 * 通用权限检查方法
 * Common check permissions method
 * @param { 权限判定 } authority
 */
function checkPermissions(authority) {
    let status = true;
    const userAuthority = getAuthority(authority);
    const currentAuthority = getAuthority();
    if (Array.isArray(userAuthority) && Array.isArray(currentAuthority)) {
        status = userAuthority.some((item) => currentAuthority.includes(item));
    }
    if (!userAuthority.length) {
        status = true;
    }
    return status;
}
exports.checkPermissions = checkPermissions;
/**
 * 检查 当前页面路由权限状态
 * @param { 路由 Matched List } matched
 */
function routerMatchedAuthority(matched = []) {
    let res = true;
    const isException = window.location.pathname.includes('/exception/');
    if (isException)
        return res;
    matched.forEach((item = {}) => {
        const { meta = {} } = item;
        const { authority } = meta;
        if (authority && !checkPermissions(authority)) {
            res = false;
        }
    });
    return res;
}
exports.routerMatchedAuthority = routerMatchedAuthority;
/**
 * 获取权限数据 并 格式化类型
 * @param { 权限内容 } authority
 */
function getAuthority(str) {
    const authorityString = typeof str === 'undefined' && localStorage ? localStorage.getItem(iAuthorityKey) : str;
    let authority;
    try {
        if (authorityString) {
            authority = JSON.parse(authorityString);
        }
    }
    catch (e) {
        authority = authorityString;
    }
    if (typeof authority === 'string' || typeof authority === 'number') {
        return [authority];
    }
    return authority;
}
exports.getAuthority = getAuthority;
/**
 * 设置存储用户权限
 * @param { 权限内容 } authority
 */
function setAuthority(authority, authorityData, router) {
    const iAuthority = typeof authority === 'string' ? [authority] : authority;
    localStorage.setItem(iAuthorityKey, JSON.stringify(iAuthority));
    window[iAuthorityKey] = authorityData;
    //暂时注释
    // if (!router) return;
    // const { matched = [] } = router.currentRoute || {};
    // if (!routerMatchedAuthority(matched)) {
    //   router.push('/exception/403');
    //   return;
    // }
}
exports.setAuthority = setAuthority;
/**
 * 获取用户路由权限
 * Common check permissions method
 * @param { 展示菜单标识 } pathKey
 * @param { 权限判定类型 } type  router | service
 * @param { 路由权限菜单，只在路由模式下生效 } menuData
 */
function getAuthMenu(pathKey, type = 'router', menuData) {
    const authorized = getAuthority();
    if (type === 'router') {
        return getMatchMenu(authorized, menuData, pathKey);
    }
    return getServiceMenu(authorized, window[iAuthorityKey], menuData, pathKey);
}
exports.getAuthMenu = getAuthMenu;
/**
 * 获取 项目配置下的 权限路由结构
 * @param { 权限列表 } authorized
 * @param { 路由配置数据 } menuData
 * @param { 需要展示的路由标识，为空则取全部数据 } pathKey
 */
function getMatchMenu(authorized, menuData = [], pathKey) {
    const matchMenuItems = [];
    if (!menuData.length)
        return [];
    // 去除无效 Menu 数据
    const nextMenuData = menuData.filter((item) => !(item.redirect && !item.children));
    // 过滤 指定 Key
    if (pathKey) {
        const curremtMenuData = nextMenuData.filter((item) => item.name === pathKey);
        if (curremtMenuData.length) {
            return getMatchMenu(authorized, curremtMenuData);
        }
        else {
            let scopeMenuData = [];
            nextMenuData.forEach((item) => {
                if (item.children && item.children.length) {
                    scopeMenuData = [...scopeMenuData, ...getMatchMenu(authorized, item.children, pathKey)];
                }
            });
            return scopeMenuData;
        }
    }
    nextMenuData.forEach((item = {}) => {
        const { authority } = item;
        if (!authority) {
            matchMenuItems.push(item);
            return;
        }
        const curremtAuthority = Array.isArray(authority) ? authority : [authority];
        if (curremtAuthority.length && !authorized.length) {
            return;
        }
        if (curremtAuthority.some((item) => authorized.includes(item))) {
            const nextItem = { ...item };
            if (item.children && item.children.length) {
                nextItem.children = getMatchMenu(authorized, item.children);
            }
            matchMenuItems.push({
                ...nextItem,
            });
        }
    });
    return matchMenuItems;
}
function listToTree(list, pid) {
    return list
        .filter((item) => item.pid === pid)
        .map((item) => ({
        ...item,
        children: listToTree(list, item.id),
    }))
        .sort((prev, next) => prev.menuOrder - next.menuOrder);
}
function treeToArray(list, newArr = []) {
    list.forEach((item) => {
        const { children } = item;
        if (children) {
            delete item.children;
            if (children.length) {
                newArr.push(item);
                return treeToArray(children, newArr);
            }
        }
        newArr.push(item);
    });
    return newArr;
}
function matchMenuByKey(menuData, key) {
    let res = [];
    const nextMenuData = menuData.filter((item = {}) => item.permitCode === key) || [];
    if (nextMenuData.length) {
        res = nextMenuData;
    }
    else {
        menuData.forEach((item) => {
            if (item.children && item.children.length) {
                res = [...res, ...matchMenuByKey(item.children, key)];
            }
        });
    }
    return res;
}
/**
 * 获取 项目配置下的 权限路由结构
 * @param { 权限列表 } authorized
 * @param { 服务权限菜单数据 } menuData
 * @param { 项目路由配置数据 } routerList
 * @param { 需要展示的路由标识，为空则取全部数据 } pathKey
 */
function getServiceMenu(authorized = [], menuData = [], routerList, pathKey) {
    let matchMenuItems = [];
    const nextRouterList = treeToArray(routerList).filter((item) => !!item.component) || [];
    menuData.forEach((item = {}) => {
        if (authorized.includes(item.permitCode)) {
            const matched = nextRouterList.filter((i) => i.path === item.component) || [];
            if (!matched.length)
                return;
            const routeItem = matched[0] || {};
            matchMenuItems.push({
                ...item,
                extends: {
                    ...routeItem,
                    matched,
                },
            });
        }
    });
    matchMenuItems = listToTree(matchMenuItems, 0);
    if (pathKey) {
        matchMenuItems = matchMenuByKey(matchMenuItems, pathKey);
    }
    return matchMenuItems;
}
const Authorized = checkPermissions;
const reloadAuthorized = async () => {
    const res = await (0, api_1.getAuthorityQuery)();
    if (res.code !== 0)
        return;
    const authorityData = res.data;
    // 初始化权限
    if (Array.isArray(authorityData)) {
        const currentAuthority = authorityData.map((item) => item.permitCode || item.component);
        setAuthority(currentAuthority, authorityData);
    }
};
exports.reloadAuthorized = reloadAuthorized;
window.reloadAuthorized = reloadAuthorized;
exports.default = Authorized;
