"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.urlReset = exports.getQueryStr = exports.setException = exports.initSecurity = exports.datetimeFormat = exports.isValidDate = exports.removeCookie = exports.cookie = exports.runFunction = exports.jsonToString = exports.trim = exports.getQueryPath = exports.getQueryString = exports.getPageQuery = exports.isNil = exports.isImg = exports.isUrl = exports.APILoader = void 0;
const qs_1 = require("qs");
const APILoader_1 = __importDefault(require("./APILoader"));
exports.APILoader = APILoader_1.default;
const max_1 = require("@umijs/max");
const base_1 = require("@teams-max/base");
const auth_1 = require("./auth");
const isUrl = (path) => {
    if (!path.startsWith('http')) {
        return false;
    }
    try {
        const url = new URL(path);
        return !!url;
    }
    catch (error) {
        return false;
    }
};
exports.isUrl = isUrl;
const isNil = (value) => value === null || value === undefined;
exports.isNil = isNil;
/** 判断是否是图片链接 */
function isImg(path) {
    return /\w.(png|jpg|jpeg|svg|webp|gif|bmp)$/i.test(path);
}
exports.isImg = isImg;
/**
 * 获取当前 URL 参数
 * @return  {Object}  处理过的 URL 参数
 */
function getPageQuery(url) {
    const href = url && isUrl(url) ? url : window.location.href;
    return (0, qs_1.parse)(href.replace('?=', '?').split('?')[1]);
}
exports.getPageQuery = getPageQuery;
/**
 * 获取地址栏参数
 * @param 参数名
 * @returns {string | null}
 */
function getQueryString(name, rmHash = true) {
    const reg = new RegExp('(^|&)' + name + '=([^&]*)(&|$)', 'i');
    const search = window.location.href.split('?')[1];
    if (search) {
        const r = search.match(reg);
        if (r !== null) {
            const res = unescape(r[2]);
            if (rmHash) {
                return res.split('#')[0];
            }
            return res;
        }
        return null;
    }
    return null;
}
exports.getQueryString = getQueryString;
function getQueryPath(path = '', query = {}) {
    const search = (0, qs_1.stringify)(query);
    if (search.length) {
        return `${path}?${search}`;
    }
    return path;
}
exports.getQueryPath = getQueryPath;
/**
 * 去掉空格
 * @param {string} 要去掉空格的字符串
 * @param   {Boolean} 是否去掉字符串中间的空格
 * @return  {String}  处理过的字符串
 */
function trim(str, isGlobal) {
    if (!str) {
        return null;
    }
    // return str.replace(/^[\s\xa0]+|[\s\xa0]+$/g, '');
    // var result = str.replace(/(^\s+)|(\s+$)/g, "");
    let result = str.replace(/^[\s\xa0]+|[\s\xa0]+$/g, '');
    if (isGlobal)
        result = result.replace(/\s/g, '');
    return result;
}
exports.trim = trim;
/**
 * 将Json数据转为String
 * @name    jsonToString
 * @param   {Object}  要转化的json对象
 * @param   {Boolean} 是否要进行转码以备URL传输
 * @return  {String}  转化后的字符串
 */
function jsonToString(json, isEncode) {
    let strTemp = '';
    for (const key in json) {
        strTemp += key + '=' + (isEncode ? encodeURIComponent(json[key]) : json[key]) + '&';
    }
    return strTemp.slice(0, -1);
}
exports.jsonToString = jsonToString;
/**
 * 如果是个方法执行一下它
 */
function runFunction(valueEnum, ...rest) {
    if (typeof valueEnum === 'function') {
        return valueEnum(...rest);
    }
    return valueEnum;
}
exports.runFunction = runFunction;
/**
 * 获取cookie和设置cookie
 * @name    cookie
 * @param   {String}  名字
 * @param   {String}  值
 * @param   {Object}  配置选项
 * @return  {String}  当只有名字时返回名字对应值
 */
function cookie(name, value, options) {
    if (typeof value !== 'undefined') {
        let nextOptions = options || {};
        let nextValue = value;
        if (value === null) {
            nextValue = '';
            nextOptions = {
                ...{},
                ...nextOptions,
                expires: -1,
            };
        }
        let expires = '';
        if (nextOptions.expires &&
            (typeof nextOptions.expires === 'number' || nextOptions.expires.toUTCString)) {
            let date;
            if (typeof nextOptions.expires === 'number') {
                date = new Date();
                date.setTime(date.getTime() + nextOptions.expires * 24 * 60 * 60 * 1000);
            }
            else {
                date = nextOptions.expires;
            }
            expires = `; expires=${date.toUTCString()}`;
        }
        const path = nextOptions.path ? `; path=${nextOptions.path}` : ';path=/';
        const domain = nextOptions.domain ? `; domain=${nextOptions.domain}` : '';
        const secure = nextOptions.secure ? '; secure' : '';
        document.cookie = [
            name,
            '=',
            encodeURIComponent(nextValue),
            expires,
            path,
            domain,
            secure,
        ].join('');
    }
    else {
        let cookienextValue = null;
        if (document.cookie && document.cookie !== '') {
            const cookies = document.cookie.split(';');
            for (let i = 0; i < cookies.length; i += 1) {
                const trimCookie = trim(cookies[i]);
                if (trimCookie.substring(0, name.length + 1) === `${name}=`) {
                    cookienextValue = decodeURIComponent(trimCookie.substring(name.length + 1));
                    break;
                }
            }
        }
        return cookienextValue;
    }
    return null;
}
exports.cookie = cookie;
/**
 * 删除cookie的快捷方法
 * @name    removeCookie
 * @param   {String}  名字
 */
function removeCookie(key) {
    cookie(key, '', { expires: -1 });
}
exports.removeCookie = removeCookie;
/**
 * Date 对象判断
 * @param {Date} value 要检查的 Date 对象
 * @returns {String} 返回Date 对象 结果
 */
function isValidDate(value) {
    const date = new Date(value);
    return date instanceof Date && !isNaN(date.valueOf());
}
exports.isValidDate = isValidDate;
/**
 * 日期时间格式化
 * @param {Date | String | Number} value 时间、类时间对象
 * @param {String} format 格式：yyyy-MM-dd hh:mm:ss:S
 * @param {Boolean} isCoerce 是否对类时间格式强制转换
 * @returns {String} 返回格式化后的值
 */
function datetimeFormat(value, format, isCoerce) {
    const isDate = isValidDate(value);
    // 不可转时间 非类似时间格式的字符串
    if ((!isDate && !isCoerce) || (!isDate && isCoerce && typeof value !== 'string')) {
        return value;
    }
    let date = new Date(value);
    // 非常规数据
    if (!isDate && isCoerce) {
        let nextValue = '';
        nextValue = value.replace(/年|月|日|_|,|，|。|\.|\//g, '-');
        nextValue = nextValue.replace(/时|分|秒|：|\//g, ':');
        const dateArr = nextValue.split(' '); // 拆分日期 时间
        const hasDay = nextValue.includes('-'); // 包含日期状态
        const hasTime = nextValue.includes(':'); // 包含时间状态
        if (dateArr.length !== 2 && !hasDay && hasTime) {
            const presentDate = new Date();
            const timeArr = nextValue.split(':').slice(0, 4);
            if (timeArr.length < 3) {
                nextValue = `${timeArr[0]}:${timeArr[1]}:${timeArr[2] || '00'}:${timeArr[3] || 0}`;
            }
            date = new Date(`${presentDate.getFullYear()}-${presentDate.getMonth() + 1}-${presentDate.getDate()} ${nextValue}`);
        }
        if (dateArr.length !== 2 && hasDay && !hasTime) {
            const nextDate = nextValue.split('-').slice(0, 3);
            if (nextDate.length === 2) {
                nextDate.unshift(new Date().getFullYear().toString());
            }
            date = new Date(`${nextDate.join('-')} 00:00:00`);
        }
        // 无法格式化时 抛出原数据
        if (!isValidDate(date)) {
            return value;
        }
    }
    if (!format) {
        return Date.parse(date);
    }
    const o = {
        'M+': date.getMonth() + 1,
        'd+': date.getDate(),
        'h+': date.getHours(),
        'm+': date.getMinutes(),
        's+': date.getSeconds(),
        'q+': Math.floor((date.getMonth() + 3) / 3),
        S: date.getMilliseconds(), // 毫秒
    };
    if (/(y+)/.test(format)) {
        // eslint-disable-next-line no-param-reassign
        format = format.replace(RegExp.$1, (date.getFullYear() + '').substr(4 - RegExp.$1.length));
    }
    const week = {
        0: '日',
        1: '一',
        2: '二',
        3: '三',
        4: '四',
        5: '五',
        6: '六',
    };
    if (/(E+)/.test(format)) {
        // eslint-disable-next-line no-param-reassign
        format = format.replace(RegExp.$1, (RegExp.$1.length > 1 ? (RegExp.$1.length > 2 ? '星期' : '周') : '') +
            week[date.getDay() + '']);
    }
    for (const k in o) {
        if (new RegExp('(' + k + ')').test(format)) {
            // eslint-disable-next-line no-param-reassign
            format = format.replace(RegExp.$1, RegExp.$1.length === 1 ? o[k] : ('00' + o[k]).substr(('' + o[k]).length));
        }
    }
    return format;
}
exports.datetimeFormat = datetimeFormat;
// 安全插件
function initSecurity(userId, userName) {
    const { securityUrl } = (0, base_1.runtimeConfig)();
    if (!userId)
        return;
    const setWaterMark = () => {
        // eslint-disable-next-line @typescript-eslint/no-unused-expressions
        window.g_waterMark && window.g_waterMark.set(`${userId} ${userName}`);
    };
    if (!window.g_waterMark) {
        new APILoader_1.default(securityUrl).load().then(() => {
            setWaterMark();
        });
        return;
    }
    setWaterMark();
}
exports.initSecurity = initSecurity;
// 设置异常状态
function setException(res, redirect) {
    const { code } = res;
    // 用于错误信息的暂存
    (0, auth_1.setErrorInfo)(code, res);
    // 10301  认证未授权
    if (max_1.history && Number(code) !== 10301) {
        const search = {
            redirect: window.location.href
        };
        max_1.history.replace({
            pathname: `/exception/${code || '404'}`,
            search: (0, qs_1.stringify)(search)
        });
    }
}
exports.setException = setException;
function getQueryStr(query, isEncode = false) {
    const queryStr = jsonToString(query || getPageQuery() || {}, isEncode);
    return queryStr && queryStr !== '' ? `?${queryStr}` : '';
}
exports.getQueryStr = getQueryStr;
// 清除 浏览器参数
function urlReset(key) {
    const query = getPageQuery() || {};
    delete query[key];
    if (key === 'ticket') {
        delete query.sso_statusid;
    }
    max_1.history.replace({
        pathname: max_1.history.location.pathname,
        hash: '',
        search: getQueryStr(query),
    });
}
exports.urlReset = urlReset;
