"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 *
 * https://developer.mozilla.org/zh-CN/docs/Web/API/WebSocket
 */
/**
 * 定义WebSocket类
 */
class WebSocketServer {
    /**
     * WebSocket类构造函数
     * @param {String} loadChartOption 加载的图表选项
     */
    constructor(url) {
        let protocol = window.location.protocol === 'http:' ? 'ws://' : 'wss://';
        this.url = `${protocol}${window.location.host}${url}`;
        this.ws = new WebSocket(this.url); // socket对象
        this.lastHealthTime = -1; // 上一次心跳时间
        this.healthMessage = 'ping'; // 心跳信息内容
        // this.bufferedAmount = this.ws.bufferedAmount; // 发送是否结束 0：发送已结束
        // this.readyState = this.ws.readyState; // 实例对象的当前状态 0：正在连接 1：连接成功 2：连接正在关闭 3：连接已关闭或者打开连接失败
        this.reconnect = 0; // 重连的时间
        this.reconnectMark = false; // 是否重连过
        this.keepAliveTimer = null; // 用来发送心跳的定时器
        this.message = null;
        this.ws.onerror = () => {
            console.log('EDRROR');
            setTimeout(() => {
                this.onclose('error');
            }, 10 * 1000);
        };
    }
    /**
     * 心跳动作
     * @returns {Object} 返回数据结果对象
     */
    keepalive() {
        let time = new Date().getTime();
        if (this.lastHealthTime !== -1 && time - this.lastHealthTime > 20 * 1000) {
            // 不是刚开始连接并且20s
            this.onclose('auto');
        }
        else if (this.ws && this.ws.bufferedAmount === 0 && this.ws.readyState === 1) {
            this.lastHealthTime = time;
            this.ws.send(this.healthMessage);
        }
    }
    /**
     * 连接失败
     * @param {String} selector 连接失败后的回调函数
     */
    onerror(fn) {
        this.ws.onerror = (event) => {
            fn(event);
            this.onclose('auto');
            console.log('EDRROR');
        };
    }
    /**
     * 连接成功后，并发送链接参数
     */
    onopen(message) {
        if (!this.ws || !message)
            return;
        this.message = message;
        // eslint-disable-next-line no-param-reassign
        message = JSON.stringify(message);
        this.ws.onopen = () => {
            this.reconnect = 0;
            this.reconnectMark = false;
            if (this.ws.readyState === 1) {
                this.ws.send(message);
                this.keepalive();
                this.keepAliveTimer = setInterval(() => {
                    this.keepalive();
                }, 2 * 1000);
            }
            // 连接已经关闭，或者打开连接失败
            if (this.ws.readyState === 3) {
                this.onclose('auto');
            }
        };
    }
    /**
     * 心跳动作
     * @returns {Object} 返回数据结果对象
     */
    heartBeat(message) {
        if (!this.ws)
            return;
        this.timmer = setTimeout(() => {
            clearTimeout(this.timmer);
            if (!this.ws)
                return;
            console.log(this.ws && this.ws.readyState);
            this.ws.send(message);
            this.heartBeat('ping');
        }, 15 * 1000);
    }
    /**
     * 从服务器接受到信息
     * @returns {Object} 返回数据结果对象
     */
    onmessage(fn) {
        if (!this.ws)
            return;
        this.ws.onmessage = function (response) {
            // 业务代码
            if (typeof response === 'object' && response.data) {
                fn(JSON.parse(response.data));
            }
            // 连接已经关闭，或者打开连接失败
            if (this.ws && this.ws.readyState === 3) {
                this.onclose('auto');
            }
        };
    }
    /**
     * 关闭指定连接
     */
    onclose(type = 'handle') {
        clearInterval(this.keepAliveTimer);
        // 手动关闭 & 抛出错误/异常 则关闭连接
        if (type === 'handle' || type === 'error') {
            this.ws.close();
            console.warn('CLOSE');
        }
        // 如果没有重连过，进行重连。
        if (!this.reconnectMark && type !== 'handle') {
            this.reconnect = new Date().getTime();
            this.reconnectMark = true;
            this.lastHealthTime = -1;
        }
        // 错误
        if (type === 'error') {
            const tempWs = this.ws; // 保存ws对象
            this.reconnect = 0;
            this.reconnectMark = false;
            this.ws = new WebSocket(this.url);
            this.ws.onerror = tempWs.onerror;
            this.onopen(this.message);
            console.log('RE OPEN');
        }
    }
}
exports.default = WebSocketServer;
